import matplotlib.pyplot as plt
import numpy as np

plt.rcParams["font.family"] = "Ubuntu"#ou autre police de caractère installée sur l'ordinateur

latitude=49.5#CHOIX UTILISATEUR
chaineLat=str(latitude).replace('.', ',') #pour affichage en degres
chaineLatTiret=str(latitude).replace('.', '-') #pour le nom du fichier
latitude=np.radians(latitude)#conversion pour calculs
eps=np.radians(23.433)#obliquité de l'écliptique

Re,  Rs=9,20#rayon équatorial,  rayon solsticial

couleur1='yellowgreen'#printemps-été
couleur2='purple'#automne-hiver

def hauteur(declinaison, H):#calcule la hauteur du Soleil (en radians)
    return np.arcsin(np.sin(latitude)*np.sin(declinaison)+np.cos(latitude)*np.cos(declinaison)*np.cos(H))

fig=plt.figure(figsize=(9, 9),
tight_layout=True,
edgecolor='royalblue', linewidth=2)
ax=plt.subplot(projection='polar')

plt.xlim(0, np.pi/2)
ax.set_theta_zero_location("S")#Angle 0 en haut
plt.ylim(0, Rs)
ax.set_yticks([])
ax.set_xticks(np.radians(np.arange(0, 90, 1)), minor=True)
ax.xaxis.grid(linewidth=0)

#graduations angles :
tick = [Rs,Rs*0.98]
for t  in np.deg2rad(np.arange(0,90,5)):
    ax.plot([t,t], tick, lw=0.72, color="k")

tick = [Rs,Rs*0.99]
for t  in np.deg2rad(np.arange(0,89,1)):
    ax.plot([t,t], tick, lw=0.72, color="k")

#arcs saisonniers :-------------------
angles=np.arange(0, np.pi/2, 0.01)
arcRe=np.ones_like(angles)*Re
plt.plot(angles, arcRe, c='C0')

#arc de déclinaison :
Rsin=Rs*np.sin(eps)
ListeRsin=np.ones_like(angles)*Rsin
ax.plot(angles, ListeRsin, c='C1')
ax.text(np.pi/2*0.98, Rsin, "arc d'obliquité  ", c='C1', ha='right', va='top', fontsize=10)
#-----------------------------------------------------
#Repère 90-latitude (pour trouver la hauteur méridienne) :
plt.plot([np.pi/2-latitude, np.pi/2-latitude], [Rs*0.95, Rs], c='slateblue')
#-----------------------------------------------------
#point orange de fixation du fil
plt.scatter([0, 0], [0, 0], c='orange')

#-----------------------------------------------------
#axe supplémentaire (ligne de visée horizontale):
ax2=ax.inset_axes([0, 1.05, 1, 0], transform=ax.transAxes)
ax2.set_xticks([])
ax2.set_yticks([])
ax2.set_title("ligne de visée")
#-----------------------------------------------------
#affichage latitude
ax.text(0.50*np.pi/2, 1, 'latitude %.4s°'%(chaineLat), c='royalblue', fontsize=13)

def lignes(heure):
    H=np.radians((heure-12)*15)
    demieH=np.radians((heure-0.5-12)*15)
    plt.plot([hauteur(eps, H), hauteur(0, H)], [Rs, Re], c=couleur1)
    plt.plot([hauteur(-eps, H), hauteur(0, H)], [Rs, Re], c=couleur2)
    plt.plot([hauteur(eps, demieH), hauteur(0, demieH)], [Rs, Re], c='gray', alpha=0.5, lw=0.5)
    plt.plot([hauteur(-eps, demieH), hauteur(0, demieH)], [Rs, Re], c='gray', alpha=0.5, lw=0.5)

#-----------------------------------------------------
#Tracé des lignes et chiffraisons horaires :
heuremin=4#5 suffira, pour les latitudes inférieures à 49°, mais3 voire 2 aux latitudes polaires
for heure in np.arange(heuremin, 12+1, 1):
    lignes(heure)
    if heure==6:#écriture décalée vers la droite
        plt.text(hauteur(0, np.radians(heure-12)*15), Re, '%i'%(heure), va='bottom', ha='left')
        plt.text(hauteur(0, np.radians(heure-12)*15), 0.95*Re, '%i'%(24-heure), va='bottom', ha='left')
    else:
        if heure>6 and heure!=12:#écriture décalée vers la gauche
            plt.text(hauteur(0, np.radians(heure-12)*15), 0.95*Re, '%i'%(24-heure), va='bottom', ha='right')
            plt.text(hauteur(0, np.radians(heure-12)*15), Re, '%i'%(heure), va='bottom', ha='right')

plt.text(hauteur(0, 0), 0.99*Re, '12', va='bottom', ha='center')#affichage du 12
#-----------------------------------------------------
#Axe inséré pour légendes des lignes horaires et du calendrier :
ax3=ax.inset_axes([0.01, 0.8, 0.2, 0.1], transform=ax.transAxes)
ax3.set_xticks([])
ax3.set_yticks([])
ax3.axis("off")
ax3.set_ylim(-3, 0)
ax3.set_xlim(0, 1)
ax3.plot([0, 0.0], [0.0, 0.0], c=couleur1, label='printemps - été')#tracé fictif, pour légendes
ax3.plot([0, 0.0], [0.0, 0.0], c=couleur2, label='automne-hiver')#tracé fictif, pour légendes
ax3.legend(bbox_to_anchor=(1.2, -0.2))#position de la légende des lignes horaires
#-----------------------------------------------------
#Calendrier :
couleurCal='firebrick'
plt.text(np.radians(91), (Re+Rs)/2, 'calendrier', c=couleurCal)
ListeDecl=np.radians([0, 11.469, 20.145, 23.43])
angle=np.ones_like(ListeDecl)*np.radians(90)

Rdelta=Rs*np.sin(eps)/(np.sin(ListeDecl)+Rs/Re*np.sin(eps-ListeDecl))
plt.scatter(angle, Rdelta, s=200, c=couleurCal, marker='|')
dates=['20 mar. - 23 sep.',
'20 avr. - 23 août - 23 oct. - 19 fév.',
'21 mai - 23 juil. - 22 nov. - 20 jan.',
'21 juin - 21 déc.']
etiqDates=['A', 'B', 'C', 'D ']
for N in range(len(ListeDecl)):
    ax.text(0.99*angle[N], Rdelta[N], etiqDates[N], c=couleurCal, ha='right', va='top')
    ax3.text(0, -N, etiqDates[N]+" : "+dates[N], c=couleurCal)
#-----------------------------------------------------
NomDuFichier="QuadrantDoubleLimbe"+chaineLatTiret
fig.savefig(NomDuFichier+".png", dpi=200)
fig.savefig(NomDuFichier+".pdf")
plt.show()
